<?php
/**
 * Picu front end template
 *
 * @since 0.3.0
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Returns template file
 *
 * @since 0.3.0
 */

function picu_load_template( $single_template ) {

	global $post;

	if ( $post->post_type == 'picu_collection' ) {
		if ( file_exists( PICU_PATH . 'frontend/picu-app.php' ) ) {
			$single_template = PICU_PATH . 'frontend/picu-app.php';
		}
	}

	return $single_template;
}

add_filter( 'single_template', 'picu_load_template', 99 );


/**
 * Gather picu body classes
 *
 * @since 0.7.3
 */
function get_picu_body_classes() {

	$picu_body_classes = array();

	// Add body class for lazyloading
	// Legacy M&C fix
	if ( defined( 'PICU_MARK_COMMENT_VERSION' ) AND version_compare( PICU_MARK_COMMENT_VERSION, '1.0.2' ) < 0 ) {
 
	} else {
		$picu_body_classes[] = 'lazyloading';
	}

	/**
	 * Add post status as body class
	 */
	global $post;

	if ( isset( $post->post_status ) AND ! empty( $post->post_status ) ) {
		$picu_body_classes[] = 'status-' . $post->post_status;
	}

	/**
	 * Add class when user is logged in
	 */
	if ( is_user_logged_in() AND current_user_can( picu_capability() ) AND apply_filters( 'picu_show_admin_bar', true ) ) {
		$picu_body_classes[] = 'has-picu-admin-bar';
	}

	/**
	 * Add filter to modify body classes
	 */
	$picu_body_classes = apply_filters( 'picu_body_classes', $picu_body_classes );

	return $picu_body_classes;

}


/**
 * Echo picu body classes
 *
 * @since 0.7.3
 */
function picu_body_classes() {

	$picu_body_classes = get_picu_body_classes();

	if ( count( $picu_body_classes ) > 0 ) {
		echo sprintf( ' class="%s"', implode( ' ', $picu_body_classes ) );
	}

}


/**
 * Load backbone templates
 *
 * @since 0.7.2
 */
function picu_load_backbone_templates() {

	$templates = array(
		'collection-info'	=> PICU_PATH . 'frontend/js/templates/picu-collection-info.php',
		'status-bar'		=> PICU_PATH . 'frontend/js/templates/picu-status-bar.php',
		'gallery-item'		=> PICU_PATH . 'frontend/js/templates/picu-gallery-item.php',
		'lightbox'			=> PICU_PATH . 'frontend/js/templates/picu-lightbox.php',
		'send-selection'	=> PICU_PATH . 'frontend/js/templates/picu-send-selection.php',
		'approved'			=> PICU_PATH . 'frontend/js/templates/picu-approved.php'
	);

	$templates = apply_filters( 'picu_load_backbone_templates', $templates );

	return $templates;
}


/**
 * Load collections, models & views
 *
 * @since 0.7.2
 */
function picu_load_cmv() {

	$cmv = array(
		'picu-state' =>				PICU_URL . 'frontend/js/models/picu-state.js?v=' . PICU_VERSION,
		'collection-info-view' =>	PICU_URL . 'frontend/js/views/collection-info-view.js?v=' . PICU_VERSION,
		'status-bar-view' =>		PICU_URL . 'frontend/js/views/status-bar-view.js?v=' . PICU_VERSION,
		'single-image' =>			PICU_URL . 'frontend/js/models/single-image.js?v=' . PICU_VERSION,
		'picu-collection' =>		PICU_URL . 'frontend/js/collections/picu-collection.js?v=' . PICU_VERSION,
		'gallery-view' =>			PICU_URL . 'frontend/js/views/gallery-view.js?v=' . PICU_VERSION,
		'single-image-view' =>		PICU_URL . 'frontend/js/views/single-image-view.js?v=' . PICU_VERSION,
		'lightbox-view' =>			PICU_URL . 'frontend/js/views/lightbox-view.js?v=' . PICU_VERSION,
		'send-selection-view' =>	PICU_URL . 'frontend/js/views/send-selection-view.js?v=' . PICU_VERSION,
		'approved-view' =>			PICU_URL . 'frontend/js/views/approved-view.js?v=' . PICU_VERSION
	);

	$cmv = apply_filters( 'picu_load_cmv', $cmv );

	return $cmv;
}


/**
 * Load front end stylesheet
 *
 * @since 0.7.0
 */
function picu_load_styles() {

	$styles = array(
		'picu' => PICU_URL . 'frontend/css/picu-dark.css',
		'picu-print' => PICU_URL . 'frontend/css/picu-print.css',
	);

	$styles = apply_filters( 'picu_load_styles', $styles );

	$styles_output = '';

	foreach( $styles as $name => $url ) {
		$styles_output .= "\n\t\t" . '<link href="' . $url . '?v=' . PICU_VERSION . '" rel="stylesheet" media="';
		if ( 'picu-print' == $name ) {
			$styles_output .= 'print';
		}
		else {
			$styles_output .= 'screen';
		}
		$styles_output .= '" />';
	}

	$styles_output = apply_filters( 'picu_styles_output', $styles_output );

	echo $styles_output . "\n";
}


/**
 * Load stylesheet depending on settings
 *
 * @since 0.7.0
 */
function picu_theme_options( $styles ) {

	$options = get_option( 'picu_settings' );

	if ( isset( $options['theme'] ) AND $options['theme'] == 'light' ) {
		$styles['picu'] = PICU_URL . 'frontend/css/picu-light.css';
	}

	return $styles;
}

add_filter( 'picu_load_styles', 'picu_theme_options', 10, 1 );


/**
 * Get images for the picu gallery
 *
 * @since 0.3.0
 *
 * @param $content, the collection content
 * @return string, javascript objects, containing the image collection
 *
 */

function picu_get_images() {

	$post = get_post();

	// Get image IDs
	$include = get_post_meta( $post->ID, '_picu_collection_gallery_ids', true );

	// Check if there are any images to build a selection from
	if ( !empty( $include ) ) {

		// Set up attachment requests
		$orderby = 'post__in';
		$order = 'ASC';
		$id = $post->ID;

		// Load attachments
		$_attachments = get_posts( array( 'include' => $include, 'post_status' => 'any', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $order, 'orderby' => $orderby ) );

		$attachments = array();
		foreach ( $_attachments as $key => $val ) {
			$attachments[$val->ID] = $_attachments[$key];
		}

		// If we don't get attachments, this is as far as we go
		if ( empty( $attachments ) ) {
			return '[ ]';
		}

		// Prepare backbone image collection
		$i = 0;
		$imgnum = 1;
		$image_collection = array();
		$orientation = '';

		// Create image objects
		foreach ( $attachments as $id => $attachment ) {

			// Calculate image orientiation
			$image_meta = wp_get_attachment_metadata( $id );

			if ( isset( $image_meta['height'], $image_meta['width'] ) ) {
				$orientation = ( $image_meta['height'] > $image_meta['width'] ) ? 'portrait' : 'landscape';
			}

			// Get image description
			if ( trim( $attachment->post_excerpt ) ) {
				$image_description = wpautop( htmlspecialchars( $attachment->post_excerpt, ENT_QUOTES, 'UTF-8' ) );
				$image_description = preg_replace( "/\r|\n/", "", $image_description );
			} else {
				$image_description = '';
			}

			// Get attachment URLs
			$image_name = wp_get_attachment_image_src( $attachment->ID, 'full' );
			$image_path = wp_get_attachment_image_src( $attachment->ID, 'picu-large' );
			$image_path_small = wp_get_attachment_image_src( $attachment->ID, 'picu-small' );

			if ( function_exists( 'wp_get_attachment_image_srcset' ) ) {
				$image_path_srcset = ( wp_get_attachment_image_srcset( $attachment->ID, 'picu-large' ) ) ?: '';
				$image_path_small_srcset = ( wp_get_attachment_image_srcset( $attachment->ID, 'picu-small' ) ) ?: '';
			}

			// Set size attribute, for use with srcset
			if ( isset( $image_meta['width'] ) ) {
				$size = $image_meta['width'] . 'px';
			}
			else {
				$size = '100vw';
			}

			// Set selected attribute, if a selection has been (auto) saved
			$temp = get_post_meta( $post->ID, '_picu_collection_selection', true );

			if ( isset( $temp['selection'] ) and is_array( $temp['selection'] ) ) {
				$selection = $temp['selection'];
				$selected = ( in_array( $attachment->ID, $selection ) ) ? true : false;
			} else {
				$selected = false;
			}

			// Get markers
			if ( isset( $temp['markers']['id_'.$id] ) AND ! empty( $temp['markers']['id_'.$id] ) ) {

				$markers = array();

				foreach( $temp['markers']['id_'.$id] as $key => $value ) {
					$markers[$key] = $value;
				}

				// See function below
				array_walk_recursive( $markers, 'picu_encode_marker_comment' );
			}
			else {
				$markers = '';
			}

			$image_title = basename( $image_name[0] );

			// Legacy M&C fix for new way of displaying image name
			// Using a regular string for M&C version older than 1.0.2
			if ( defined( 'PICU_MARK_COMMENT_VERSION' ) AND version_compare( PICU_MARK_COMMENT_VERSION, '1.0.2' ) < 0 ) {
				$title = $image_title;
			}
			else {
				$title = array( 'number' => $imgnum, 'filename' => $image_title );
			}

			$current_image = array(
				'number' => $imgnum,
				'imageID' => $attachment->ID,
				'title' => $title,
				'description' => $image_description,
				'imagePath' => $image_path[0],
				'imagePath_small' => $image_path_small[0],
				'imagePath_srcset' => $image_path_srcset,
				'imagePath_small_srcset' => $image_path_small_srcset,
				'size' => $size,
				'orientation' => $orientation,
				'selected' => $selected,
				'markers' => $markers
			);

			/*
			 * Add filter to modify each individual image
			 */
			$current_image = apply_filters( 'picu_single_image_data', $current_image );

			$image_collection[] = array(
				'number' => $current_image['number'],
				'imageID' => $current_image['imageID'],
				'title' => $current_image['title'],
				'description' => $current_image['description'],
				'imagePath' => $current_image['imagePath'],
				'imagePath_small' => $current_image['imagePath_small'],
				'imagePath_srcset' => $current_image['imagePath_srcset'],
				'imagePath_small_srcset' => $current_image['imagePath_small_srcset'],
				'size' => $current_image['size'],
				'orientation' => $current_image['orientation'],
				'selected' => $current_image['selected'],
				'markers' => $current_image['markers']
			);

			$imgnum++;
		}

		return json_encode( $image_collection, JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT );

	}
	else {
		return '[ ]';
	}
}



/**
 * Helper function to add slashes and encode comments
 */
function picu_encode_marker_comment( &$item, $key ) {
	if ( $key == 'comment' ) {
		$item = addslashes( htmlspecialchars( $item, ENT_QUOTES, 'UTF-8' ) );
	}
}



/**
 * Create AppState JSON object
 *
 * @since 0.6.0
 *
 * @param $id, collection id
 * @return string, json
 *
 */

function picu_get_app_state() {

	$post = get_post();
	$id = $post->ID;

	$description = wpautop ( htmlspecialchars( get_post_meta( $id, '_picu_collection_description', true ), ENT_QUOTES, 'UTF-8' ) );

	$state = array(
		'version' => PICU_VERSION,
		'nonce' => wp_create_nonce( 'picu-ajax-security' ),
		'postid' => get_the_ID( $id ),
		'poststatus' => get_post_status( $id ),
		'title' => get_the_title( $id ),
		'date' => get_the_date( get_option( 'date_format' ), $id ),
		'description' => preg_replace( "/\r|\n/", "", $description ),
		'ajaxurl' => admin_url( 'admin-ajax.php' ),
		'error-msg-no-imgs' => __( '<h2>No images found</h2><p>It seems there are no images in this collection.</p>', 'picu' ),
		'error-msg-filter-selected' => __( 'You have not selected any images.', 'picu' ),
		'error-msg-filter-unselected' => __( 'You have no <em>unselected</em> images.', 'picu' ),
		'reset-filter-msg' => __( 'Reset filter to show all images', 'picu' )
	);


	/**
	 * Add filter to extend on app state parameters
	 */
	$state = apply_filters( 'picu_app_state', $state );

	$app_state = json_encode( $state, JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT );

	return $app_state;

}


/**
 * Collection list shortcode
 *
 * @since 1.2.0
 *
 * @param $atts
 * @return html
 *
 */

function picu_list_collections( $atts ) {

	// Setup default arguments
	$args = array(
		'post_type' => 'picu_collection',
		'orderby' => 'date',
		'posts_per_page' => -1
	);

	// Show approved or sent only?
	if ( is_array( $atts ) AND in_array( 'show_approved', $atts ) ) {
		$args['post_status'] = array( 'approved', 'sent' );
	}
	else {
		$args['post_status'] = 'sent';
	}

	// Check for IDs
	if ( isset( $atts['ids'] ) and ! empty( $atts['ids'] ) ) {
		$ids = explode( ',', $atts['ids'] );
		$ids = array_map( 'trim', $ids );
		$args['post__in'] = $ids;
	}

	// Check for email address
	if ( isset( $atts['email'] ) and ! empty( $atts['email'] ) ) {
		$email = sanitize_email( $atts['email'] );

		$args['meta_query'] = array(
			'relation' => 'AND',
			array(
				'key' => '_picu_collection_email_address',
				'value' => $email
			)
		);
	}

	// Filter query args
	$args = apply_filters( 'picu_list_collections_args', $args );

	$query = new WP_Query( $args );

	$collection_list = '<ul class="picu-collection-list">';

	while ( $query->have_posts() ) {
		$query->the_post();
		$collection_list .= '<li class="picu-status-' . get_post_status() . '"><a href="' . get_permalink() . '">' . get_the_title() . '</a></li>';
	}

	wp_reset_query();

	$collection_list .= '</ul>';

	return $collection_list;
}

add_shortcode( 'picu_list_collections', 'picu_list_collections' );